#include <unistd.h>
#include <signal.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "linkface_sdk.h"
#include "log_util.h"

#define PRODUCT_SECRET      "cl77brMh9ysIUQQA"
int g_flag_is_running = 1;

void sig_handler(int signo) {
    LOG_INFO("catch the signal SIGINT %d\n", signo);
    g_flag_is_running = 0;
}

static int del_all_face(void *arg) {
    LOG_INFO("delete all done\r\n");
    return 0;
}

static int add_person(const struct iotx_person_t *person, void *arg) {
    LOG_INFO("add face to local database: %s\n", person->id);
    for (int i = 0; i < person->face_feature_size; ++i) {
        LOG_INFO("face feature: %s\n", person->face_feature[i]);
    }
    return 0;
}

static int del_person(const char *person_id, void *user) {
    LOG_INFO("del face from local database: %s\n", person_id);
    return 0;
}

extern int linkkit_init_adapter(struct iotx_lf_config_t *cfg);

extern int linkkit_destroy_adapter();

int main(int argc, char **argv) {
    char *pk = NULL;
    char *dn = NULL;
    char *ds = NULL;
    char *path = NULL;
    char *log_path = NULL;
    if (argc < 5) {
        LOG_WARN(
                "please run like this: ./link_face_demo a1AqJTqFF1W huaanTest4 ddddddddddddddddddddssssssssssss path log_path\n");
        return 0;
    }
    pk = argv[1];
    dn = argv[2];
    ds = argv[3];
    path = argv[4];
    log_path = argv[5];

    LOG_INFO("args: %s %s %s %s %s\n", pk, dn, ds, path, log_path);
    if (signal(SIGINT, sig_handler) == SIG_ERR) {
        perror("[FACE-SDK] signal error.\n");
        exit(EXIT_FAILURE);
    }
    register_add_person(add_person, NULL);
    register_delete_person(del_person, NULL);
    register_delete_all(del_all_face, NULL);

    char *mqtt_url = "iot-as-mqtt.cn-shanghai.aliyuncs.com";
    char *http2_url = "iot-as-http2.cn-shanghai.aliyuncs.com";
    struct iotx_log_cfg_t log_cfg = {
            .lvl = LF_LOG_INFO,
            .dst = LF_FILE,
            .log_path = log_path,
            .max_size_log_mb = 10
    };
    struct iotx_lf_config_t config = {
            .public_instance = 1,
            .mqtt_url = mqtt_url,
            .http2_url = http2_url,
            .product_key = pk,
            .device_name = dn,
            .product_secret = PRODUCT_SECRET,
            .device_secret = ds,
            .data_path = path,
            .cfg_log = log_cfg,
            .update_mode = 1
    };

    _retry:
#ifdef ADAPTER_MODE
    LOG_INFO("adapter mode init.\n");
    if (1 != linkkit_init_adapter(&config)) {
        LOG_ERROR("[FACE-SDK] init failed, retry 3s later.\n");
        sleep(3);
        goto _retry;
    }

    if (1 != iotx_lf_init(&config)) {
        LOG_INFO("init failed, retry 3s later.\n");
        sleep(3);
        iotx_lf_deinit();
        linkkit_destroy_adapter();
        goto _retry;
    }
#else
    LOG_INFO("singleton mode init.\n");
    if (1 != iotx_lf_init(&config)) {
        LOG_INFO("iotx_lf_init failed, retry 3s later.\n");
        sleep(3);
        iotx_lf_deinit();
        goto _retry;
    }
#endif
    LOG_INFO("link face sdk init done.\n");

    // 事件上报case
    struct iotx_matche_result_t realtime_res = {
            .panorama_img = "case1",
            .panorama_img_name = "test_realtime",
            .panorama_img_size = 7,
            .device_id = "device_id",
            .match_status = MATCH_STATUS_FAILED,
            .is_realtime = 1,
    };
    struct iotx_matche_result_t match_realtime_res = {
            .panorama_img = "case2",
            .panorama_img_name = "test_match_realtime",
            .panorama_img_size = 7,
            .device_id = "device_id",
            .match_status = MATCH_STATUS_SUCCESS,
            .is_realtime = 1,
            .person_id = "lhbpbph129q3prod"
    };
    struct iotx_matche_result_t none_realtime_res = {
            .panorama_img = "case3",
            .panorama_img_name = "test_none_realtime",
            .panorama_img_size = 7,
            .device_id = "device_id",
            .is_realtime = 0,
            .time_msec = 1626871517000,
            .match_status = MATCH_STATUS_FAILED
    };
    struct iotx_matche_result_t match_none_realtime_res = {
            .panorama_img = "case4",
            .panorama_img_name = "test_match_none_realtime",
            .panorama_img_size = 7,
            .device_id = "device_id",
            .match_status = MATCH_STATUS_SUCCESS,
            .is_realtime = 0,
            .time_msec = 1626871517000,
            .person_id = "lhbpbph129q3prod"
    };

    // 保持设备连接状态
    while (1 == g_flag_is_running) {
        sleep(30);
        int ret = upload_match_result(&realtime_res);
        LOG_INFO("realtime_ret %d\r\n", ret);
        ret = upload_match_result(&match_realtime_res);
        LOG_INFO("match_realtime_ret %d\r\n", ret);
        ret = upload_match_result(&none_realtime_res);
        LOG_INFO("none_realtime_ret %d\r\n", ret);
        ret = upload_match_result(&match_none_realtime_res);
        LOG_INFO("match_none_realtime %d\r\n", ret);
        none_realtime_res.time_msec += 5000;
        match_none_realtime_res.time_msec += 5000;
    }

    // 单次上报事件
    int ret = upload_match_result(&realtime_res);
    LOG_INFO("realtime_ret %d\r\n", ret);
    ret = upload_match_result(&match_realtime_res);
    LOG_INFO("match_realtime_ret %d\r\n", ret);
    ret = upload_match_result(&none_realtime_res);
    LOG_INFO("none_realtime_ret %d\r\n", ret);
    ret = upload_match_result(&match_none_realtime_res);
    LOG_INFO("match_none_realtime %d\r\n", ret);

#ifdef ADAPTER_MODE
    LOG_INFO("adapter mode destroy.\n");
    linkkit_destroy_adapter();
    iotx_lf_deinit();
#else
    LOG_INFO("singleton mode destroy.\n");
    iotx_lf_deinit();
#endif
    return 0;
}
